'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.User = undefined;

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _find = require('lodash/collection/find');

var _find2 = _interopRequireDefault(_find);

var _icons = require('@recipher/icons');

var _navigation = require('@recipher/navigation');

var _table = require('@recipher/table');

var _permissions = require('../../data/permissions');

var _permissions2 = _interopRequireDefault(_permissions);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var Permission = function Permission(_ref) {
  var permission = _ref.permission,
      member = _ref.member,
      user = _ref.user,
      onRemove = _ref.onRemove,
      onAdd = _ref.onAdd;

  var items = [{ Icon: _icons.Menu, name: user.id, title: permission.description, align: 'right',
    style: { dropdown: { top: 26, right: 0 } },
    submenu: _permissions2.default.map(function (p) {
      return { title: p.description,
        claim: 'manage ' + member.id,
        onClick: function onClick(_) {
          return onAdd(member, user, p.right);
        },
        active: permission.right === p.right };
    })
  }];

  items[0].submenu.push({ divider: true, claim: 'manage ' + member.id });
  items[0].submenu.push({ title: 'Revoke Access ',
    claim: 'manage ' + member.id,
    onClick: function onClick(_) {
      return onRemove(member, user);
    }, active: false });

  return _react2.default.createElement(_navigation.Menu, { items: items, style: { margin: 0 } });
};

var Action = function Action(_ref2) {
  var member = _ref2.member,
      owner = _ref2.owner,
      user = _ref2.user,
      existing = _ref2.existing,
      onAdd = _ref2.onAdd,
      onRemove = _ref2.onRemove;

  var permission = existing && (0, _find2.default)(_permissions2.default, { right: existing.claim.right });

  var add = [{ title: 'Add User', Icon: _icons.Menu, onClick: function onClick(_) {
      return onAdd(member, user);
    }, claim: 'manage ' + member.id }];

  return _react2.default.createElement(
    'span',
    null,
    owner && 'Owner',
    !owner && permission && _react2.default.createElement(Permission, { permission: permission, user: existing, member: member, onAdd: onAdd, onRemove: onRemove }),
    !owner && !existing && _react2.default.createElement(_navigation.Menu, { items: add, style: { margin: 0 } })
  );
};

var User = exports.User = function User(_ref3) {
  var member = _ref3.member,
      user = _ref3.user,
      users = _ref3.users,
      onRemove = _ref3.onRemove,
      onAdd = _ref3.onAdd,
      state = _ref3.state,
      dispatch = _ref3.dispatch,
      $ = _ref3.$;

  var events = (0, _table.Events)({ dispatch: dispatch, $: $, id: user.id });

  var owner = member.createdBy === user.id,
      existing = (0, _find2.default)(users.data, { id: user.id });

  var onClick = !owner && !existing ? function (_) {
    return onAdd(member, user);
  } : null,
      style = !owner && !existing ? {} : { cursor: 'default' };

  return _react2.default.createElement(
    _table.Row,
    { onMouseOver: events.over, onMouseOut: events.out, onClick: onClick, style: style },
    _react2.default.createElement(
      _table.Cell,
      { width: '65%' },
      _react2.default.createElement(
        'strong',
        null,
        user.name,
        ' '
      ),
      _react2.default.createElement(
        'span',
        { style: { fontSize: '0.9em', color: '#666' } },
        user.email
      )
    ),
    _react2.default.createElement(
      _table.Cell,
      { width: '35%', align: 'right', hide: !owner && !existing && !state[user.id] },
      _react2.default.createElement(Action, { owner: owner, existing: existing, user: user, member: member, onAdd: onAdd, onRemove: onRemove })
    )
  );
};

exports.default = (0, _table.State)('user')(User);