'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.accountsSelector = exports.RESET = exports.DISCONNECT_FAILED = exports.DISCONNECT_SUCCESS = exports.DISCONNECT = exports.CONNECT_FAILED = exports.CONNECT_SUCCESS = exports.CONNECT = exports.CONNECTING = exports.FETCH_FAILED = exports.FETCH_SUCCESS = exports.FETCH = exports.schema = undefined;
exports.default = reducer;
exports.fetch = fetch;
exports.list = list;
exports.connect = connect;
exports.disconnect = disconnect;
exports.connectToFacebook = connectToFacebook;
exports.connectTo = connectTo;
exports.connectToTwitter = connectToTwitter;
exports.connectToGoogle = connectToGoogle;
exports.connectRss = connectRss;

var _assign = require('lodash/object/assign');

var _assign2 = _interopRequireDefault(_assign);

var _union = require('lodash/array/union');

var _union2 = _interopRequireDefault(_union);

var _without = require('lodash/array/without');

var _without2 = _interopRequireDefault(_without);

var _isArray = require('lodash/lang/isArray');

var _isArray2 = _interopRequireDefault(_isArray);

var _resource = require('@recipher/resource');

var _entities = require('@recipher/entities');

var _members = require('./members');

var _sessionWeb = require('@recipher/session-web');

var _normalizr = require('normalizr');

var _support = require('@recipher/support');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var schema = exports.schema = new _normalizr.Schema('accounts');

var FETCH = exports.FETCH = 'recipher/member/accounts/FETCH';
var FETCH_SUCCESS = exports.FETCH_SUCCESS = 'recipher/member/accounts/FETCH_SUCCESS';
var FETCH_FAILED = exports.FETCH_FAILED = 'recipher/member/accounts/FETCH_FAILED';

var CONNECTING = exports.CONNECTING = 'recipher/member/accounts/CONNECTING';
var CONNECT = exports.CONNECT = 'recipher/member/accounts/CONNECT';
var CONNECT_SUCCESS = exports.CONNECT_SUCCESS = 'recipher/member/accounts/CONNECT_SUCCESS';
var CONNECT_FAILED = exports.CONNECT_FAILED = 'recipher/member/accounts/CONNECT_FAILED';

var DISCONNECT = exports.DISCONNECT = 'recipher/member/accounts/DISCONNECT';
var DISCONNECT_SUCCESS = exports.DISCONNECT_SUCCESS = 'recipher/member/accounts/DISCONNECT_SUCCESS';
var DISCONNECT_FAILED = exports.DISCONNECT_FAILED = 'recipher/member/accounts/DISCONNECT_FAILED';

var RESET = exports.RESET = 'recipher/member/accounts/RESET';

var initialState = {
  error: null,
  isFetching: false,
  isFetched: false,
  isConnecting: false,
  isDisconnecting: false,
  data: []
};

var ERRORS = {
  500: 'Server error',
  422: 'Account already in use'
};

function reducer() {
  var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : initialState;
  var action = arguments[1];

  switch (action.type) {
    case FETCH:
      return (0, _assign2.default)({}, state, { isFetching: true, isFetched: false });

    case FETCH_SUCCESS:
      return (0, _assign2.default)({}, state, { data: action.payload.result, isFetching: false, isFetched: true });

    case FETCH_FAILED:
      return (0, _assign2.default)({}, state, { isFetching: false, isFetched: true });

    case CONNECTING:
      return (0, _assign2.default)({}, state, { isConnecting: true });

    case CONNECT_SUCCESS:
      var result = (0, _isArray2.default)(action.payload.result) ? action.payload.result : [action.payload.result];
      return (0, _assign2.default)({}, state, { isConnecting: false, data: (0, _union2.default)(result, state.data) });

    case CONNECT_FAILED:
      return (0, _assign2.default)({}, state, { error: ERRORS[action.payload.status], isConnecting: false });

    case DISCONNECT:
      return (0, _assign2.default)({}, state, { isDisconnecting: true });

    case DISCONNECT_SUCCESS:
      return (0, _assign2.default)({}, state, { isDisconnecting: false, data: (0, _without2.default)(state.data, action.meta.account.id) });

    case DISCONNECT_FAILED:
      return (0, _assign2.default)({}, state, { error: ERRORS[action.payload.status], isDisconnecting: false });

    case RESET:
    case _sessionWeb.SIGN_OUT_SUCCESS:
      return initialState;

    default:
      return state;
  }
};

var accountsSelector = exports.accountsSelector = (0, _entities.entitiesSelector)(function (state) {
  return state.member.accounts;
}, 'accounts');

function fetch(member) {
  if (member == null) return;

  return _defineProperty({}, _resource.RESOURCE, {
    types: [FETCH, FETCH_SUCCESS, FETCH_FAILED],
    payload: {
      url: '/members/' + member.id + '/accounts',
      method: 'get',
      query: { limit: 100, sort: 'network' },
      normalize: function normalize(r) {
        return (0, _normalizr.normalize)(r.accounts, (0, _normalizr.arrayOf)(schema));
      }
    }
  });
};

function list(member) {
  return function (dispatch, getState) {
    var state = accountsSelector(getState());

    if (state.data.length) return;

    return dispatch(fetch(member));
  };
};

function connect(member, network, account) {
  account.network = network;

  return _defineProperty({}, _resource.RESOURCE, {
    types: [CONNECT, CONNECT_SUCCESS, CONNECT_FAILED],
    payload: {
      url: '/members/' + member.id + '/accounts',
      method: 'post',
      data: { account: account },
      normalize: function normalize(r) {
        return r.accounts ? (0, _normalizr.normalize)(r.accounts, (0, _normalizr.arrayOf)(schema)) : (0, _normalizr.normalize)(r.account, schema);
      }
    }
  });
};

function disconnect(account) {
  return _defineProperty({}, _resource.RESOURCE, {
    types: [DISCONNECT, DISCONNECT_SUCCESS, DISCONNECT_FAILED],
    payload: {
      url: '/accounts/' + account.id,
      method: 'del'
    },
    meta: { account: account }
  });
};

function connectToFacebook() {
  return function (dispatch, getState) {
    var _getState = getState(),
        facebook = _getState.session.facebook;

    dispatch({ type: CONNECTING });

    facebook.login().then(function (authentication) {
      dispatch(connect((0, _members.memberSelector)(getState()), 'facebook', authentication));
    }, function (error) {
      dispatch({ type: CONNECT_FAILED, payload: { statusText: '', status: 400 } });
    });
  };
};

function connectTo(network) {
  var WIDTH = { google: 900, twitter: 600 };

  if (network === 'facebook') return connectToFacebook();

  return function (dispatch, getState) {
    if (network === 'rss') return;

    (0, _support.dialog)('/auth/' + network, { title: network, width: WIDTH[network] }).then(function (authentication) {
      dispatch(connect((0, _members.memberSelector)(getState()), network, authentication));
    }, function (error) {
      dispatch({ type: CONNECT_FAILED, payload: { statusText: '', status: 400 } });
    });
  };
};

function connectToTwitter() {
  return connectTo('twitter');
};

function connectToGoogle() {
  return connectTo('google');
};

function connectRss(data) {
  return function (dispatch, getState) {
    dispatch(connect((0, _members.memberSelector)(getState()), 'rss', data));
  };
};