'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.connectionsSelector = exports.RESET = exports.REMOVE_FAILED = exports.REMOVE_SUCCESS = exports.REMOVE = exports.ADD_FAILED = exports.ADD_SUCCESS = exports.ADD = exports.FETCH_FAILED = exports.FETCH_SUCCESS = exports.FETCH = exports.schema = undefined;
exports.default = reducer;
exports.fetch = fetch;
exports.search = search;
exports.list = list;
exports.change = change;
exports.add = add;
exports.remove = remove;

var _assign = require('lodash/object/assign');

var _assign2 = _interopRequireDefault(_assign);

var _union = require('lodash/array/union');

var _union2 = _interopRequireDefault(_union);

var _without = require('lodash/array/without');

var _without2 = _interopRequireDefault(_without);

var _isArray = require('lodash/lang/isArray');

var _isArray2 = _interopRequireDefault(_isArray);

var _resource = require('@recipher/resource');

var _entities = require('@recipher/entities');

var _members = require('./members');

var _sessionWeb = require('@recipher/session-web');

var _normalizr = require('normalizr');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var schema = exports.schema = new _normalizr.Schema('connections');

var FETCH = exports.FETCH = 'recipher/member/connections/FETCH';
var FETCH_SUCCESS = exports.FETCH_SUCCESS = 'recipher/member/connections/FETCH_SUCCESS';
var FETCH_FAILED = exports.FETCH_FAILED = 'recipher/member/connections/FETCH_FAILED';

var ADD = exports.ADD = 'recipher/member/connections/ADD';
var ADD_SUCCESS = exports.ADD_SUCCESS = 'recipher/member/connections/ADD_SUCCESS';
var ADD_FAILED = exports.ADD_FAILED = 'recipher/member/connections/ADD_FAILED';

var REMOVE = exports.REMOVE = 'recipher/member/connections/REMOVE';
var REMOVE_SUCCESS = exports.REMOVE_SUCCESS = 'recipher/member/connections/REMOVE_SUCCESS';
var REMOVE_FAILED = exports.REMOVE_FAILED = 'recipher/member/connections/REMOVE_FAILED';

var RESET = exports.RESET = 'recipher/member/connections/RESET';

var initialState = {
  fetching: false,
  fetched: false,
  connecting: false,
  disconnecting: false,
  data: [],
  error: null,
  meta: {
    query: {
      limit: _entities.LIMIT
    }
  }
};

var ERRORS = {
  500: 'Server error'
};

function reducer() {
  var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : initialState;
  var action = arguments[1];

  switch (action.type) {
    case FETCH:
      return (0, _assign2.default)({}, state, { fetching: true, fetched: false });

    case FETCH_SUCCESS:
      return (0, _assign2.default)({}, state, { data: action.payload.result, fetching: false, fetched: true });

    case FETCH_FAILED:
      return (0, _assign2.default)({}, state, { fetching: false, fetched: true });

    case ADD:
      return (0, _assign2.default)({}, state, { connecting: true });

    case ADD_SUCCESS:
      return (0, _assign2.default)({}, state, { connecting: false, data: (0, _union2.default)([action.payload.result], state.data) });

    case ADD_FAILED:
      return (0, _assign2.default)({}, state, { error: ERRORS[action.payload.status], connecting: false });

    case REMOVE:
      return (0, _assign2.default)({}, state, { disconnecting: true });

    case REMOVE_SUCCESS:
      return (0, _assign2.default)({}, state, { disconnecting: false, data: (0, _without2.default)(state.data, action.meta.connection.id) });

    case REMOVE_FAILED:
      return (0, _assign2.default)({}, state, { error: ERRORS[action.payload.status], disconnecting: false });

    case RESET:
    case _sessionWeb.SIGN_OUT_SUCCESS:
      return initialState;

    default:
      return state;
  }
};

var connectionsSelector = exports.connectionsSelector = (0, _entities.entitiesSelector)(function (state) {
  return state.member.connections;
}, 'connections', 'connectedTo.name');

function fetch(member) {
  var query = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : { page: 1, limit: _entities.LIMIT };

  if (member == null) return;

  return _defineProperty({}, _resource.RESOURCE, {
    types: [FETCH, FETCH_SUCCESS, FETCH_FAILED],
    payload: {
      url: '/members/' + member.id + '/connections',
      method: 'get',
      query: query,
      normalize: function normalize(r) {
        return (0, _normalizr.normalize)(r.connections, (0, _normalizr.arrayOf)(schema));
      }
    },
    meta: { query: query }
  });
};

function search(query) {
  return function (dispatch, getState) {
    var member = (0, _members.memberSelector)(getState());

    return dispatch(fetch(member, query));
  };
};

function list(member) {
  return function (dispatch, getState) {
    var state = connectionsSelector(getState());

    if (state.data.length) return;

    return dispatch(fetch(member));
  };
};

function change(member, connection, type) {
  return function (dispatch, getState) {
    dispatch(remove(member, connection));
    dispatch(add(member, { memberId: connection.connectedTo.id, type: type }));
  };
};

function add(member, connection) {
  return _defineProperty({}, _resource.RESOURCE, {
    types: [ADD, ADD_SUCCESS, ADD_FAILED],
    payload: {
      url: '/members/' + member.id + '/connections',
      method: 'post',
      data: { connection: connection },
      normalize: function normalize(r) {
        return (0, _normalizr.normalize)(r.connection, schema);
      }
    }
  });
};

function remove(member, connection) {
  return _defineProperty({}, _resource.RESOURCE, {
    types: [REMOVE, REMOVE_SUCCESS, REMOVE_FAILED],
    payload: {
      url: '/members/' + member.id + '/connections/' + connection.connectedTo.id,
      query: { type: connection.type },
      method: 'del'
    },
    meta: { connection: connection }
  });
};