'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.memberSelector = exports.primarySelector = exports.membersSelector = exports.SET_PRIMARY_FAILED = exports.SET_PRIMARY_SUCCESS = exports.SET_PRIMARY = exports.UPDATE_FAILED = exports.UPDATE_SUCCESS = exports.UPDATE = exports.CREATE_FAILED = exports.CREATE_SUCCESS = exports.CREATE = exports.FETCH_FAILED = exports.FETCH_SUCCESS = exports.FETCH = exports.schema = undefined;
exports.default = reducer;
exports.fetch = fetch;
exports.create = create;
exports.update = update;
exports.setPrimary = setPrimary;

var _assign = require('lodash/object/assign');

var _assign2 = _interopRequireDefault(_assign);

var _union = require('lodash/array/union');

var _union2 = _interopRequireDefault(_union);

var _find = require('lodash/collection/find');

var _find2 = _interopRequireDefault(_find);

var _reselect = require('reselect');

var _resource = require('@recipher/resource');

var _entities = require('@recipher/entities');

var _sessionWeb = require('@recipher/session-web');

var _normalizr = require('normalizr');

var _support = require('@recipher/support');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var schema = exports.schema = new _normalizr.Schema('members');

var FETCH = exports.FETCH = 'recipher/member/member/FETCH';
var FETCH_SUCCESS = exports.FETCH_SUCCESS = 'recipher/member/member/FETCH_SUCCESS';
var FETCH_FAILED = exports.FETCH_FAILED = 'recipher/member/member/FETCH_FAILED';

var CREATE = exports.CREATE = 'recipher/member/member/CREATE';
var CREATE_SUCCESS = exports.CREATE_SUCCESS = 'recipher/member/member/CREATE_SUCCESS';
var CREATE_FAILED = exports.CREATE_FAILED = 'recipher/member/member/CREATE_FAILED';

var UPDATE = exports.UPDATE = 'recipher/member/member/UPDATE';
var UPDATE_SUCCESS = exports.UPDATE_SUCCESS = 'recipher/member/member/UPDATE_SUCCESS';
var UPDATE_FAILED = exports.UPDATE_FAILED = 'recipher/member/member/UPDATE_FAILED';

var SET_PRIMARY = exports.SET_PRIMARY = 'recipher/member/member/SET_PRIMARY';
var SET_PRIMARY_SUCCESS = exports.SET_PRIMARY_SUCCESS = 'recipher/member/member/SET_PRIMARY_SUCCESS';
var SET_PRIMARY_FAILED = exports.SET_PRIMARY_FAILED = 'recipher/member/member/SET_PRIMARY_FAILED';

var ERRORS = {
  403: 'Not authorized',
  500: 'Error fetching member'
};

var initialState = {
  data: [],
  primary: null,
  error: null,
  changing: false
};

function reducer() {
  var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : initialState;
  var action = arguments[1];

  switch (action.type) {
    case FETCH:
      return (0, _assign2.default)({}, state);
    case FETCH_SUCCESS:
      var members = action.payload.entities.members,
          member = (0, _find2.default)(members, { primary: true }),
          primary = member ? member.id : action.payload.result[0];


      return (0, _assign2.default)({}, state, { data: action.payload.result, primary: primary, error: null });

    case UPDATE:
      return (0, _assign2.default)({}, state, { error: null, changing: true });

    case UPDATE_SUCCESS:
    case CREATE_SUCCESS:
      var owned = !action.meta.notOwned;
      return (0, _assign2.default)({}, state, { data: owned ? (0, _union2.default)([action.payload.result], state.data) : state.data,
        primary: owned && action.payload.result,
        error: null, changing: false });

    case FETCH_FAILED:
    case CREATE_FAILED:
    case UPDATE_FAILED:
      return (0, _assign2.default)({}, state, { error: ERRORS[action.payload.status], changing: false });

    case SET_PRIMARY_SUCCESS:
      return (0, _assign2.default)({}, state, { primary: action.payload.result, error: null });

    case _sessionWeb.SIGN_OUT_SUCCESS:
      return initialState;
    default:
      return state;
  }
};

var membersSelector = exports.membersSelector = (0, _entities.entitiesSelector)(function (state) {
  return state.member.members;
}, 'members');

var primarySelector = exports.primarySelector = function primarySelector(state) {
  return state.member.members.primary;
};

var memberSelector = exports.memberSelector = (0, _reselect.createSelector)([primarySelector, membersSelector], function (primary, members) {
  return (0, _find2.default)(members.data, { id: primary }) || members.data[0];
});

function fetch(user) {
  if (user == null) return;

  return _defineProperty({}, _resource.RESOURCE, {
    types: [FETCH, FETCH_SUCCESS, FETCH_FAILED],
    payload: {
      url: '/members?userid=' + user.id,
      method: 'get',
      normalize: function normalize(r) {
        return (0, _normalizr.normalize)(r.members, (0, _normalizr.arrayOf)(schema));
      }
    }
  });
};

function create(data) {
  if (data.smla === true) data.license = (0, _support.config)('license').toString();

  return _defineProperty({}, _resource.RESOURCE, {
    types: [CREATE, CREATE_SUCCESS, CREATE_FAILED],
    payload: {
      url: '/members',
      method: 'post',
      data: { member: data },
      normalize: function normalize(r) {
        return (0, _normalizr.normalize)(r.member, schema);
      }
    },
    meta: data
  });
};

function update(member, data) {
  var license = (0, _support.config)('license').toString();
  if (member.license !== license) data.license = license;

  return _defineProperty({}, _resource.RESOURCE, {
    types: [UPDATE, UPDATE_SUCCESS, UPDATE_FAILED],
    payload: {
      url: '/members/' + member.id,
      method: 'put',
      data: { member: data },
      normalize: function normalize(r) {
        return (0, _normalizr.normalize)(r.member, schema);
      }
    },
    meta: { data: data }
  });
};

function setPrimary(member, user) {
  return _defineProperty({}, _resource.RESOURCE, {
    types: [SET_PRIMARY, SET_PRIMARY_SUCCESS, SET_PRIMARY_FAILED],
    payload: {
      url: '/members/' + member.id + '/users/' + user.id,
      method: 'put',
      normalize: function normalize(r) {
        return (0, _normalizr.normalize)(r.member, schema);
      }
    }
  });
};