'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.usersSelector = exports.RESET = exports.INVITE_FAILED = exports.INVITE_SUCCESS = exports.INVITE = exports.REMOVE_FAILED = exports.REMOVE_SUCCESS = exports.REMOVE = exports.ADD_FAILED = exports.ADD_SUCCESS = exports.ADD = exports.FETCH_FAILED = exports.FETCH_SUCCESS = exports.FETCH = exports.schema = undefined;
exports.default = reducer;
exports.fetch = fetch;
exports.search = search;
exports.list = list;
exports.add = add;
exports.invite = invite;
exports.remove = remove;

var _assign = require('lodash/object/assign');

var _assign2 = _interopRequireDefault(_assign);

var _union = require('lodash/array/union');

var _union2 = _interopRequireDefault(_union);

var _without = require('lodash/array/without');

var _without2 = _interopRequireDefault(_without);

var _authorize = require('@recipher/authorize');

var _authorize2 = _interopRequireDefault(_authorize);

var _resource = require('@recipher/resource');

var _entities = require('@recipher/entities');

var _members = require('./members');

var _sessionWeb = require('@recipher/session-web');

var _normalizr = require('normalizr');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var schema = exports.schema = new _normalizr.Schema('users');

var FETCH = exports.FETCH = 'recipher/member/users/FETCH';
var FETCH_SUCCESS = exports.FETCH_SUCCESS = 'recipher/member/users/FETCH_SUCCESS';
var FETCH_FAILED = exports.FETCH_FAILED = 'recipher/member/users/FETCH_FAILED';

var ADD = exports.ADD = 'recipher/member/users/ADD';
var ADD_SUCCESS = exports.ADD_SUCCESS = 'recipher/member/users/ADD_SUCCESS';
var ADD_FAILED = exports.ADD_FAILED = 'recipher/member/users/ADD_FAILED';

var REMOVE = exports.REMOVE = 'recipher/member/users/REMOVE';
var REMOVE_SUCCESS = exports.REMOVE_SUCCESS = 'recipher/member/users/REMOVE_SUCCESS';
var REMOVE_FAILED = exports.REMOVE_FAILED = 'recipher/member/users/REMOVE_FAILED';

var INVITE = exports.INVITE = 'recipher/member/users/INVITE';
var INVITE_SUCCESS = exports.INVITE_SUCCESS = 'recipher/member/users/INVITE_SUCCESS';
var INVITE_FAILED = exports.INVITE_FAILED = 'recipher/member/users/INVITE_FAILED';

var RESET = exports.RESET = 'recipher/member/users/RESET';

var initialState = {
  fetching: false,
  fetched: false,
  inviting: false,
  data: [],
  error: null,
  meta: {
    query: {
      limit: _entities.LIMIT
    }
  }
};

var ERRORS = {
  500: 'Server error'
};

function reducer() {
  var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : initialState;
  var action = arguments[1];

  switch (action.type) {
    case FETCH:
      return (0, _assign2.default)({}, state, { fetching: true, fetched: false, error: null });

    case FETCH_SUCCESS:
      return (0, _assign2.default)({}, state, { data: action.payload.result, fetching: false, fetched: true, meta: action.meta });

    case FETCH_FAILED:
      return (0, _assign2.default)({}, state, { fetching: false, fetched: false, error: ERRORS[action.payload.status] });

    case INVITE:
      return (0, _assign2.default)({}, state, { inviting: true });

    case ADD_SUCCESS:
    case INVITE_SUCCESS:
      return (0, _assign2.default)({}, state, { inviting: false, data: (0, _union2.default)(state.data, [action.payload.result]) });

    case REMOVE_SUCCESS:
      return (0, _assign2.default)({}, state, { data: (0, _without2.default)(state.data, action.meta.user.id) });

    case ADD_FAILED:
    case REMOVE_FAILED:
    case INVITE_FAILED:
      return (0, _assign2.default)({}, state, { inviting: false, fetching: false, error: ERRORS[action.payload.status] });

    case _sessionWeb.SIGN_OUT_SUCCESS:
      return initialState;

    default:
      return state;
  }
};

var usersSelector = exports.usersSelector = (0, _entities.entitiesSelector)(function (state) {
  return state.member.users;
}, 'users');

function fetch(member) {
  var query = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : { page: 1, limit: _entities.LIMIT };

  if (member == null) return;

  return _defineProperty({}, _resource.RESOURCE, {
    types: [FETCH, FETCH_SUCCESS, FETCH_FAILED],
    payload: {
      url: '/members/' + member.id + '/users',
      method: 'get',
      query: query,
      normalize: function normalize(r) {
        return (0, _normalizr.normalize)(r.users, (0, _normalizr.arrayOf)(schema));
      }
    },
    meta: { query: query }
  });
};

function search(query) {
  return function (dispatch, getState) {
    var member = (0, _members.memberSelector)(getState());

    return dispatch(fetch(member, query));
  };
};

function list(member) {
  return function (dispatch, getState) {
    var state = usersSelector(getState());

    if (state.data.length) return;

    return dispatch(fetch(member));
  };
};

function add(member, user, right) {
  if (right == null) right = _authorize2.default.permissions.ReadOnly;

  return _defineProperty({}, _resource.RESOURCE, {
    types: [ADD, ADD_SUCCESS, ADD_FAILED],
    payload: {
      url: '/members/' + member.id + '/users',
      method: 'post',
      data: { user: { id: user.id }, right: right },
      normalize: function normalize(r) {
        return (0, _normalizr.normalize)(r.user, schema);
      }
    }
  });
};

function invite(member, email, right) {
  if (right == null) right = _authorize2.default.permissions.ReadOnly;

  return _defineProperty({}, _resource.RESOURCE, {
    types: [INVITE, INVITE_SUCCESS, INVITE_FAILED],
    payload: {
      url: '/members/' + member.id + '/users',
      method: 'post',
      data: { user: { email: email }, right: right },
      normalize: function normalize(r) {
        return (0, _normalizr.normalize)(r.user, schema);
      }
    },
    meta: { member: member, email: email }
  });
};

function remove(member, user) {
  if (member.createdBy === user.id) return;

  return _defineProperty({}, _resource.RESOURCE, {
    types: [REMOVE, REMOVE_SUCCESS, REMOVE_FAILED],
    payload: {
      url: '/members/' + member.id + '/users/' + user.id,
      method: 'del'
    },
    meta: { user: user }
  });
};