'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createMember = createMember;
exports.update = update;
exports.signedUp = signedUp;
exports.setPrimary = setPrimary;
exports.createFailed = createFailed;
exports.signedOn = signedOn;
exports.reloaded = reloaded;
exports.verified = verified;

var _assign = require('lodash/object/assign');

var _assign2 = _interopRequireDefault(_assign);

var _frieze = require('frieze');

var _reactRouterRedux = require('react-router-redux');

var _reduxSaga = require('redux-saga');

var _effects = require('redux-saga/effects');

var _members = require('../ducks/members');

var _accounts = require('../ducks/accounts');

var _sessionWeb = require('@recipher/session-web');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var _marked = [fetchMember, createMember, update, signedUp, setPrimary, createFailed, signedOn, reloaded, verified].map(regeneratorRuntime.mark);

function fetchMember() {
  return regeneratorRuntime.wrap(function fetchMember$(_context) {
    while (1) {
      switch (_context.prev = _context.next) {
        case 0:
          _context.next = 2;
          return (0, _effects.select)(function (state) {
            return state.session.user;
          });

        case 2:
          _context.t0 = _context.sent;
          _context.t1 = (0, _members.fetch)(_context.t0);
          _context.next = 6;
          return (0, _effects.put)(_context.t1);

        case 6:
        case 'end':
          return _context.stop();
      }
    }
  }, _marked[0], this);
};

function createMember() {
  var members, user;
  return regeneratorRuntime.wrap(function createMember$(_context2) {
    while (1) {
      switch (_context2.prev = _context2.next) {
        case 0:
          if (!true) {
            _context2.next = 14;
            break;
          }

          _context2.next = 3;
          return (0, _effects.take)(_members.FETCH_SUCCESS);

        case 3:
          _context2.next = 5;
          return (0, _effects.select)(_members.membersSelector);

        case 5:
          members = _context2.sent;

          if (!(members.data.length === 0)) {
            _context2.next = 12;
            break;
          }

          _context2.next = 9;
          return (0, _effects.select)(function (state) {
            return state.session.user;
          });

        case 9:
          user = _context2.sent;
          _context2.next = 12;
          return (0, _effects.put)((0, _members.create)({ userId: user.id, name: user.name }));

        case 12:
          _context2.next = 0;
          break;

        case 14:
        case 'end':
          return _context2.stop();
      }
    }
  }, _marked[1], this);
};

function update() {
  var _ref, meta, message;

  return regeneratorRuntime.wrap(function update$(_context3) {
    while (1) {
      switch (_context3.prev = _context3.next) {
        case 0:
          if (!true) {
            _context3.next = 10;
            break;
          }

          _context3.next = 3;
          return (0, _effects.take)(_members.UPDATE_SUCCESS);

        case 3:
          _ref = _context3.sent;
          meta = _ref.meta;
          message = meta.data.accepted ? 'You have successfully signed up.' : 'The member information has been updated.';
          _context3.next = 8;
          return (0, _effects.put)((0, _frieze.info)(message));

        case 8:
          _context3.next = 0;
          break;

        case 10:
        case 'end':
          return _context3.stop();
      }
    }
  }, _marked[2], this);
};

function signedUp() {
  var _ref2, _meta, session, primary, data;

  return regeneratorRuntime.wrap(function signedUp$(_context4) {
    while (1) {
      switch (_context4.prev = _context4.next) {
        case 0:
          if (!true) {
            _context4.next = 25;
            break;
          }

          _context4.next = 3;
          return (0, _effects.take)(_members.CREATE_SUCCESS);

        case 3:
          _ref2 = _context4.sent;
          _meta = _ref2.meta;
          _context4.next = 7;
          return (0, _effects.select)(function (state) {
            return state.session;
          });

        case 7:
          session = _context4.sent;
          _context4.next = 10;
          return (0, _effects.select)(_members.primarySelector);

        case 10:
          primary = _context4.sent;

          if (!session.user) {
            _context4.next = 16;
            break;
          }

          _context4.next = 14;
          return (0, _effects.put)((0, _reactRouterRedux.push)('/member'));

        case 14:
          _context4.next = 21;
          break;

        case 16:
          _context4.next = 18;
          return (0, _effects.select)(function (state) {
            return state.signup.data;
          });

        case 18:
          data = _context4.sent;
          _context4.next = 21;
          return (0, _effects.put)((0, _sessionWeb.signOn)(data));

        case 21:
          _context4.next = 23;
          return (0, _effects.put)((0, _frieze.info)('You have successfully signed up ' + _meta.name + '.'));

        case 23:
          _context4.next = 0;
          break;

        case 25:
        case 'end':
          return _context4.stop();
      }
    }
  }, _marked[3], this);
};

function setPrimary() {
  var member;
  return regeneratorRuntime.wrap(function setPrimary$(_context5) {
    while (1) {
      switch (_context5.prev = _context5.next) {
        case 0:
          if (!true) {
            _context5.next = 10;
            break;
          }

          _context5.next = 3;
          return (0, _effects.take)(_members.SET_PRIMARY_SUCCESS);

        case 3:
          _context5.next = 5;
          return (0, _effects.select)(_members.memberSelector);

        case 5:
          member = _context5.sent;
          _context5.next = 8;
          return (0, _effects.put)((0, _frieze.info)('You are now managing ' + member.name + '.'));

        case 8:
          _context5.next = 0;
          break;

        case 10:
        case 'end':
          return _context5.stop();
      }
    }
  }, _marked[4], this);
};

function createFailed() {
  var session;
  return regeneratorRuntime.wrap(function createFailed$(_context6) {
    while (1) {
      switch (_context6.prev = _context6.next) {
        case 0:
          if (!true) {
            _context6.next = 10;
            break;
          }

          _context6.next = 3;
          return (0, _effects.take)(_members.CREATE_FAILED);

        case 3:
          _context6.next = 5;
          return (0, _effects.select)(function (state) {
            return state.session;
          });

        case 5:
          session = _context6.sent;
          _context6.next = 8;
          return (0, _effects.put)((0, _frieze.error)('There was an error during sign up. Please try again.'));

        case 8:
          _context6.next = 0;
          break;

        case 10:
        case 'end':
          return _context6.stop();
      }
    }
  }, _marked[5], this);
};

function signedOn() {
  return regeneratorRuntime.wrap(function signedOn$(_context7) {
    while (1) {
      switch (_context7.prev = _context7.next) {
        case 0:
          return _context7.delegateYield((0, _reduxSaga.takeLatest)(_sessionWeb.SIGN_ON_SUCCESS, fetchMember), 't0', 1);

        case 1:
        case 'end':
          return _context7.stop();
      }
    }
  }, _marked[6], this);
};

function reloaded() {
  return regeneratorRuntime.wrap(function reloaded$(_context8) {
    while (1) {
      switch (_context8.prev = _context8.next) {
        case 0:
          return _context8.delegateYield((0, _reduxSaga.takeLatest)(_sessionWeb.RELOAD_SUCCESS, fetchMember), 't0', 1);

        case 1:
        case 'end':
          return _context8.stop();
      }
    }
  }, _marked[7], this);
};

function verified() {
  return regeneratorRuntime.wrap(function verified$(_context9) {
    while (1) {
      switch (_context9.prev = _context9.next) {
        case 0:
          return _context9.delegateYield((0, _reduxSaga.takeLatest)(_sessionWeb.VERIFY_SUCCESS, fetchMember), 't0', 1);

        case 1:
        case 'end':
          return _context9.stop();
      }
    }
  }, _marked[8], this);
};